package com.sifu.sfcc.controller;

import com.sifu.sfcc.common.api.CommonPage;
import com.sifu.sfcc.common.api.CommonResult;
import com.sifu.sfcc.model.CcLabel;
import com.sifu.sfcc.service.CcLabelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@Api(tags = "CcLabelController", description = "标签")
@RequestMapping("/label")
public class CcLabelController  {

    @Autowired
    CcLabelService labelService;

    @ApiOperation(value = "添加标签")
    @PostMapping(value = "/add")
    public CommonResult add(@RequestBody CcLabel label){
    	
    	if("".equals(label.getName()) || null == label.getName()) {
    		return CommonResult.failed("标签名称不能为空");
    	}
    	
    	if(null != label.getName() && label.getName().length() > 255) {
    		return CommonResult.failed("标签名称字数不超过255个字符");
    	}
    	if(null != label.getPos() && label.getPos().length() > 255) {
    		return CommonResult.failed("标签位置字数不超过255个字符");
    	}
        int rows = labelService.add(label);
        if(rows > 0){
            return CommonResult.success(null , "成功");
        }else{
            return CommonResult.failed();
        }
    }

    @ApiOperation(value = "修改标签")
    @PostMapping(value = "/update")
    public CommonResult update(@RequestBody CcLabel label){
        int rows = labelService.update(label);
        if(rows > 0){
            return CommonResult.success(null , "成功");
        }else{
            return CommonResult.failed();
        }
    }


    @ApiOperation(value = "删除标签")
    @PostMapping(value = "/delete/{id}")
    public CommonResult delete(@PathVariable Long id){
        int rows = labelService.delete(id);
        if(rows > 0){
            return CommonResult.success(null , "成功");
        }else{
            return CommonResult.failed();
        }
    }
    
    @ApiOperation(value = "批量删除标签")
    @RequestMapping(value = "/batchDelete", method = RequestMethod.POST)
    public CommonResult batchDelete(@RequestParam("ids") List<Long> ids){
    
        int rows = labelService.batchDelete(ids);
        if(rows > 0){
            return CommonResult.success(null , "成功");
        }else{
            return CommonResult.failed();
        }
    }
    
    @ApiOperation(value = "关闭和显示用户")
    @RequestMapping(value = "/updateStatus/{id}", method = RequestMethod.POST)
    public CommonResult updateStatus(@PathVariable Long id , @ApiParam("是否显示 ： 0 - 显示  1 - 不显示 ") @RequestParam Integer status){
    
        int rows = labelService.updateStatus(id,status);
        if(rows > 0){
            return CommonResult.success(null , "成功");
        }else{
            return CommonResult.failed();
        }
    }

    @ApiOperation(value = "获取某个标签")
    @GetMapping(value = "/get/{id}")
    public CommonResult get(@PathVariable Long id){
        return CommonResult.success(labelService.get(id));
    }
    
    @ApiOperation(value = "获取标签")
    @GetMapping(value = "/list")
    public CommonResult list(@ApiParam("名称") @RequestParam(value = "name" , required = false) String name ,
    						@ApiParam("是否显示 ： 0 - 显示  1 - 不显示 ") @RequestParam(value = "status" , defaultValue = "2") Integer status ,
                             @RequestParam(value = "pageSize", defaultValue = "5") Integer pageSize,
                             @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum){
        return CommonResult.success(CommonPage.restPage(labelService.list(name ,status, pageSize ,pageNum)));
    }


}
