package com.sifu.sfcc.controller;

import com.alibaba.excel.util.CollectionUtils;
import com.alibaba.excel.util.StringUtils;
import com.sifu.sfcc.common.api.CommonPage;
import com.sifu.sfcc.common.api.CommonResult;
import com.sifu.sfcc.common.exception.ApiException;
import com.sifu.sfcc.common.annotation.CcOperateLog;
import com.sifu.sfcc.dto.CcApproveParam;
import com.sifu.sfcc.dto.CcDealerParam;
import com.sifu.sfcc.dto.CcWorderLabelParam;
import com.sifu.sfcc.dto.CcWorkOrderCount;
import com.sifu.sfcc.dto.CcWorkOrderParam;
import com.sifu.sfcc.enums.ApproveStatusEnum;
import com.sifu.sfcc.enums.WorderStatusEnum;
import com.sifu.sfcc.model.*;
import com.sifu.sfcc.service.CcLabelService;
import com.sifu.sfcc.service.CcWorkOrderService;
import com.sifu.sfcc.service.UmsAdminService;
import com.sifu.sfcc.service.UmsDeptService;
import com.sifu.sfcc.util.ValidatorUtils;
import com.sifu.sfcc.vo.workorder.WorkOrderVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;


@Api(tags = "CcWorkOrderController", description = "工单接口")
@RestController
@RequestMapping("/worder")
public class CcWorkOrderController {

    public Logger logger = LoggerFactory.getLogger(CcWorkOrderController.class);

    @Autowired
    private CcWorkOrderService ccWorkOrderService;

    @Autowired
    private CcLabelService labelService;

    @Autowired
    private UmsAdminService umsAdminService;

    @Autowired
    private UmsDeptService deptService;

    @ApiOperation(value = "创建工单")
    @PostMapping(value = "/create")
    @CcOperateLog(type = "创建", moudule = "工单")
    public CommonResult<CcWorkOrder> create(@RequestBody CcWorkOrderParam ccWorkOrderParam) throws IllegalAccessException {
        ValidatorUtils.validateEntity(ccWorkOrderParam);

        CcWorkOrder ccWorkOrder = ccWorkOrderService.create(ccWorkOrderParam);
        if (!ObjectUtils.isEmpty(ccWorkOrder) && !StringUtils.isEmpty(ccWorkOrder.getCode())) {
            return CommonResult.success(ccWorkOrder);
        } else {
            return CommonResult.failed();
        }
    }
    @ApiOperation(value = "修改工单普通字段，不修改的字段设置 null ，工单编号必填|除 （工单负责人字段、工单状态）")
    @PostMapping(value = "/makeLabel")
    @CcOperateLog(moudule = "工单", type = "添加标签")
    public CommonResult<CcWorkOrder> makeLabel(@RequestBody CcWorderLabelParam param) throws IllegalAccessException {
        ValidatorUtils.validateEntity(param);
        WorkOrderVO workOrderVO = null;
        try{
            workOrderVO = ccWorkOrderService.qryByOrderNo(param.getWorderNo());
           // String oldLabel = workOrderVO.getLabel();
            //String newLabel = StringUtils.isEmpty(oldLabel)?param.getLabel():oldLabel.concat(",").concat(param.getLabel());
            workOrderVO.setLabel(param.getLabel());
            CcWorkOrder ccWorkOrder = new CcWorkOrder();
            ccWorkOrder.setLabel(param.getLabel());
            ccWorkOrder.setCode(param.getWorderNo());
            ccWorkOrderService.updWorder(ccWorkOrder);
            //去掉标签插入
//            CcLabel ccLabel = new CcLabel();
//            ccLabel.setName(param.getLabel());
//            ccLabel.setPos(workOrderVO.getId().toString());
//            labelService.add(ccLabel);
        }catch(Exception e){
            throw new ApiException("创建工单 标签失败！");
        }

        return CommonResult.success(workOrderVO);
    }

    @ApiOperation(value = "修改工单普通字段，不修改的字段设置 null ，工单编号必填|除 （工单负责人字段、工单状态）")
    @PostMapping(value = "/changeNormal")
    @CcOperateLog(moudule = "工单", type = "修改")
    public CommonResult changeNormal(@RequestBody CcWorkOrder workOrderParam) {
        if (StringUtils.isEmpty(workOrderParam.getCode())) {
            throw new ApiException("工单编号不能为空");
        }
        int row = ccWorkOrderService.updWorder(workOrderParam);
        if (row > 0) {
            return CommonResult.success(row);
        }

        return CommonResult.failed("修改记录不存在");
    }

    @Deprecated
    @PostMapping("/changeDealeer")
    @ApiOperation(value = "更改负责人")
    @CcOperateLog(moudule = "工单", type = "修改负责人")
    public CommonResult<String> changeDealer(@RequestBody CcDealerParam dealerParam) throws IllegalAccessException {
        ValidatorUtils.validateEntity(dealerParam);
        try {
            ccWorkOrderService.transferWOrder(dealerParam.getWorderNo(), dealerParam.getDealerCode());
        } catch (ApiException e) {
            e.printStackTrace();
            logger.error("系统内部错误:" + e.getMessage());
            return CommonResult.failed("系统内部错误:" + e.getMessage());
        } catch (Exception e) {
            logger.error("未知错误{}", e);
            return CommonResult.failed("未知错误");
        }

        return CommonResult.success("成功");
    }

    @PostMapping("/cancelOrder/{worderCode}")
    @ApiOperation(value = "作废工单")
    @CcOperateLog(moudule = "工单", type = "作废工单")
    @Transactional(rollbackFor = Exception.class)
    public CommonResult<String> cancelOrder(@PathVariable("worderCode") String worderCode) throws IllegalAccessException {
        try {
            ccWorkOrderService.cancelWorder(worderCode);
        } catch (ApiException e) {
            e.printStackTrace();
            logger.error("系统内部错误:" + e.getMessage());
            return CommonResult.failed("系统内部错误:" + e.getMessage());
        } catch (Exception e) {
            logger.error("未知错误{}", e);
            return CommonResult.failed("未知错误");
        }

        return CommonResult.success("成功");
    }


    @ApiOperation(value = "通过工单编号查询工单")
    @GetMapping(value = "/getByNO/{worderCode}")
    public CommonResult<WorkOrderVO> getByNO(@PathVariable("worderCode") String wOrderNO) throws IllegalAccessException {
        WorkOrderVO workOrderVO = ccWorkOrderService.qryByOrderNo(wOrderNO);
        if (CollectionUtils.isEmpty(workOrderVO.getApproveList())) {
            CommonResult.failed("当前工单编号对应的工单不存在!");
        }
        return CommonResult.success(workOrderVO);
    }

    @ApiOperation(value = "处理工单")
    @PostMapping(value = "/process")
    public CommonResult approveOrder(@RequestBody CcApproveParam approveParam) {
        try {
            ccWorkOrderService.processWorder(approveParam);
        } catch (ApiException e) {
            return CommonResult.failed(e.getMessage());

        } catch (Exception e) {
            e.printStackTrace();
            return CommonResult.failed("处理工单异常,未知错误!");
        }

        return CommonResult.success("");
        //ccWorkOrderService.

    }

    @ApiOperation(value = "分页获取工单")
    @GetMapping(value = "/list")
    public CommonResult list(
            @RequestParam(value = "pageSize", defaultValue = "5") Integer pageSize,
            @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
            @ApiParam("工单状态 0初始化 1:处理中 2结单 3 作废")
            @RequestParam(value = "wstatus") Integer wstatus,
            @ApiParam("工单待处理人")
            @RequestParam(value = "toDoText") String toDoText
            ) {
        return CommonResult.success(CommonPage.restPage(ccWorkOrderService.list(pageSize, pageNum,wstatus,toDoText)));
    }

    @PostMapping(value = "/qryToDO")
    @ApiOperation(value = "查询待办")
    public CommonResult<List<CcWorkOrder>> qryToDO() {
        return CommonResult.success(ccWorkOrderService.toDOList());

    }
    
    @PostMapping(value = "/getCount")
    @ApiOperation(value = "查询工单个数")
    public CommonResult<List<CcWorkOrderCount>> getCount(
    		@RequestParam(value = "startTime", required = false) String startTime,
            @RequestParam(value = "endTime", required = false) String endTime,
            @RequestParam(value = "dealerId", required = false) Long dealerId
            ) {
        return CommonResult.success(ccWorkOrderService.getCount(startTime,endTime,dealerId));

    }


}
